'use strict';

var assert = require('assert');
var fs = require('fs');
var SSLGenerator = require('../');
var https = require('https');

var Store = function MemoryStore() {
  var cache  = {};
  this.get = function (domain, fn) {
    fn(null, cache[domain] || null);
  };

  this.set = function (certKeyData, fn) {
    cache[certKeyData.domain] = {
      cert: certKeyData.cert,
      key: certKeyData.key
    };
    fn();
  };
};

var generator = new SSLGenerator({
  ca: {
    cert: fs.readFileSync('./test/ca.crt.pem'),
    key: fs.readFileSync('./test/ca.key.pem')
  },
  csr: {
    country: 'US',
    state: 'Texas',
    locality: 'Longview',
    organization: 'Northshore Network Solutions',
    organizationUnit: '',
    emailAddress: 'support@northshore.io'
  },
  ttl: 10000,
  store: new Store(),
});

generator.selfSigned('*.northshore.io', function (err, certKey) {
  if (err) {
    console.error(err.stack);
  }
  https.createServer({
    cert: certKey.cert,
    key: certKey.key
  }, function (req, res) {
    res.end('Hello world!');
  }).listen(2000);
});